#include "client_window.h"

/* Lancement de la fenetre GTK, et du thread de mise a jour de celle-ci */
void client_window(WinUpdateData* updateData) {
	g_thread_init(NULL);
	gdk_threads_init();

	g_type_init();
	gtk_init(NULL, NULL);

	// Creation des elements de la fenetre
	client_window_create(updateData);

	// Lancement du thread charge de recuperer les configurations de la
	// fenetre et de la faire mettre a jour
	g_thread_create((void *) client_updates, updateData, FALSE, NULL);

	gtk_window_fullscreen(GTK_WINDOW(updateData->window));
	gtk_widget_show_all(updateData->window);

	// Mise a jour de la fenetre (premiere configuration)
	client_window_update(updateData);

	gdk_threads_enter();
	gtk_main();
	gdk_threads_leave();
}

/* Creation des elements de la fenetre */
void client_window_create(WinUpdateData* updateData) {
	GtkWidget *quitBtn;
	gint screenWidth, screenHeight;
	PangoFontDescription* font;

	// Creation
	updateData->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(updateData->window), TITLE);

	// Ajout d'un layout fixe
	updateData->layout = gtk_layout_new(NULL, NULL);
	gtk_container_add(GTK_CONTAINER(updateData->window), updateData->layout);

	// Creation des elements graphiques
	updateData->screenshotImg = gtk_image_new();
	updateData->numberImg = gtk_image_new();
	updateData->label = gtk_label_new("");

	// Creation d'un label, utile dans certaines configurations
	font = pango_font_description_from_string("Monospace");
	pango_font_description_set_size(font, 40 * PANGO_SCALE);
	gtk_widget_modify_font(updateData->label, font);

	// Bouton permettant de quitter, en haut a gauche
	quitBtn = gtk_button_new();
	gtk_container_add(GTK_CONTAINER(quitBtn), gtk_image_new_from_stock(GTK_STOCK_QUIT, GTK_ICON_SIZE_LARGE_TOOLBAR));

	// Recuperation des dimensions de l'ecran
	screenWidth = gdk_screen_get_width(gdk_screen_get_default());
	screenHeight = gdk_screen_get_height(gdk_screen_get_default());

	// Placements des elements graphiques sur la fenetre (en pourcentage)
	gtk_layout_put(GTK_LAYOUT(updateData->layout), updateData->screenshotImg, screenWidth * 0.1 / 2, screenHeight * 0.15 / 2);
	gtk_layout_put(GTK_LAYOUT(updateData->layout), updateData->numberImg, screenWidth * 0.82, screenHeight * 0.02);
	gtk_layout_put(GTK_LAYOUT(updateData->layout), updateData->label, (screenWidth - 300) / 2, (screenHeight - 100) / 2);
	gtk_layout_put(GTK_LAYOUT(updateData->layout), quitBtn, 10, 10);

	// Signaux
	g_signal_connect(G_OBJECT(quitBtn), "clicked", G_CALLBACK(gtk_main_quit), NULL);
	g_signal_connect(G_OBJECT(updateData->window), "destroy", G_CALLBACK(gtk_main_quit), NULL);

	gtk_widget_show_all(quitBtn);
}

/* Mise a jour de l'affichage de la fenetre, selon la configuration passee */
void client_window_update(WinUpdateData* updateData) {
	GdkPixbuf *screenshotImgBuf;
	GdkColor color;
	gint screenWidth, screenHeight;

	char screenshotFilename[256],
		numberFilename[256],
		timerFilename[256],
		playerName[256];

	// Pour chacun des evements possibles (indique le type de configuration
	// passe, a appliquer)
	switch (updateData->winClient.event) {

	// Configuration de jeu classique (affichage d'une serie)
	case EVENT_PLAY:
		sprintf(numberFilename, "data/interface/number-%i.png", updateData->winClient.numberStrum);

		sprintf(screenshotFilename,
			"data/series/%s/screen_%i.png",
			updateData->winClient.serieDir,
			updateData->winClient.screenNumber
		);

		// Taille de l'ecran
		screenWidth = gdk_screen_get_width(gdk_screen_get_default());
		screenHeight = gdk_screen_get_height(gdk_screen_get_default());

		// Mise a jour de la couleur de fond
		gdk_color_parse(updateData->winClient.color, &color);
		gtk_widget_modify_bg(updateData->layout, GTK_STATE_NORMAL, &color);

		// Affichage de l'image de la serie, chargee a la bonne taille,
		// selon l'ecran
		screenshotImgBuf = gdk_pixbuf_new_from_file_at_size(screenshotFilename, screenWidth * 0.9, screenHeight * 0.9, NULL);
		gtk_image_set_from_pixbuf(GTK_IMAGE(updateData->screenshotImg), screenshotImgBuf);

		// Nombre de strums 
		gtk_image_set_from_file(GTK_IMAGE(updateData->numberImg), numberFilename);

		gtk_widget_show(updateData->screenshotImg);
		gtk_widget_show(updateData->numberImg);
		gtk_widget_hide(updateData->label);
		break;

	// Configuration d'un compte a rebours (affichage d'un chiffre)
	case EVENT_COUNTDOWN:
		gdk_color_parse("black", &color);
		gtk_widget_modify_bg(updateData->layout, GTK_STATE_NORMAL, &color);

		sprintf(timerFilename, "data/interface/countdown_%i.png", updateData->winClient.screenNumber);
		gtk_image_set_from_file(GTK_IMAGE(updateData->screenshotImg), timerFilename);

		gtk_widget_show(updateData->screenshotImg);
		gtk_widget_hide(updateData->numberImg);
		gtk_widget_hide(updateData->label);
		break;

	// Configuration d'une fenetre validee a juste titre
	case EVENT_GOOD:
		gdk_color_parse("green", &color);

	// Configuration d'un coup de vibrato justifie
	case EVENT_GOOD_WHAMMY_BAR:
		if (updateData->winClient.event != EVENT_GOOD) {
			gdk_color_parse("green", &color);
			gtk_widget_modify_fg(updateData->label, GTK_STATE_NORMAL, &color);

			sprintf(playerName, "BONUS SWITCH\n  Joueur %d", updateData->winClient.downBy);

			gdk_color_parse("black", &color);
		}

	// Configuration d'une fenetre validee par erreur
	case EVENT_BAD:
		if (updateData->winClient.event == EVENT_BAD)
			gdk_color_parse("red", &color);

		gtk_widget_modify_bg(updateData->layout, GTK_STATE_NORMAL, &color);

		if (updateData->winClient.event != EVENT_GOOD_WHAMMY_BAR) {
			gdk_color_parse("black", &color);
			gtk_widget_modify_fg(updateData->label, GTK_STATE_NORMAL, &color);
			sprintf(playerName, "Joueur %d", updateData->winClient.downBy);
		}

		gtk_label_set_text(GTK_LABEL(updateData->label), playerName);

		gtk_widget_hide(updateData->screenshotImg);
		gtk_widget_hide(updateData->numberImg);
		gtk_widget_show(updateData->label);
		break;

	// Configuration d'une fenetre qui n'a pas encore recu de mise a jour
	case EVENT_FIRST:
		gdk_color_parse("black", &color);
		gtk_widget_modify_bg(updateData->layout, GTK_STATE_NORMAL, &color);

		gtk_image_set_from_file(GTK_IMAGE(updateData->screenshotImg), "data/interface/title.png");
		gtk_widget_hide(updateData->numberImg);
		break;

	// Configuration d'une fenetre de time out
	case EVENT_TIMER:
		gdk_color_parse("black", &color);
		gtk_widget_modify_bg(updateData->layout, GTK_STATE_NORMAL, &color);

		gdk_color_parse("red", &color);
		gtk_widget_modify_fg(updateData->label, GTK_STATE_NORMAL, &color);

		gtk_label_set_text(GTK_LABEL(updateData->label), "Time out !");

		gtk_widget_hide(updateData->screenshotImg);
		gtk_widget_hide(updateData->numberImg);
		gtk_widget_show(updateData->label);
		break;

	// Configuration d'une fenetre de pause
	case EVENT_PAUSE:
		gdk_color_parse("black", &color);
		gtk_widget_modify_bg(updateData->layout, GTK_STATE_NORMAL, &color);

		gtk_image_set_from_file(GTK_IMAGE(updateData->screenshotImg), "data/interface/pause.png");

		gtk_widget_show(updateData->screenshotImg);
		gtk_widget_hide(updateData->numberImg);
		gtk_widget_hide(updateData->label);
		break;

	// On demande a quitter le jeu
	case EVENT_QUIT:
		MGBT_EXIT_SIGNAL = 1;
		gtk_main_quit();

		break;
	}
}
