#include "main_window.h"

/* Lancement d'une fenetre graphique pour initialiser le jeu */
void main_window(GameStatus* game, Clients* clients) {
	MainWindowData mainWindowData;
	MainWindowServerData mainWindowServerData;

	mainWindowServerData.mainWindowData = &mainWindowData;

	g_type_init();
	g_thread_init(NULL);
	gdk_threads_init();
	gtk_init(NULL, NULL);

	// Creation
	mainWindowData.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(mainWindowData.window), TITLE);
	gtk_window_set_resizable(GTK_WINDOW(mainWindowData.window), FALSE);

	// Icone
	gtk_window_set_icon(GTK_WINDOW(mainWindowData.window),
			gdk_pixbuf_new_from_file("data/interface/icon.png", NULL));

	// Fenetre toujours au centre de l'ecran
	gtk_window_set_position(GTK_WINDOW(mainWindowData.window),
			GTK_WIN_POS_CENTER_ALWAYS);

	gtk_container_set_border_width(GTK_CONTAINER(mainWindowData.window), 10);
	g_signal_connect(G_OBJECT(mainWindowData.window), "destroy",
			G_CALLBACK(gtk_main_quit), NULL);

	// Preparation de la structure de donnee qui va transiter de fonction en fonction
	mainWindowData.game = game;
	mainWindowData.clients = clients;

	// Par defaut, c'est le launcher (choix entre client ou serveur) qui est lance
	main_window_launcher(&mainWindowServerData);

	gdk_threads_enter();
	gtk_main();
	gdk_threads_leave();
}

/* Creation du launcher */
void main_window_launcher(MainWindowServerData* mainWindowServerData) {
	MainWindowData* mainWindowData = mainWindowServerData->mainWindowData;
	GtkWidget *clientButton, *serverButton, *helpButton, *aboutButton,
			*quitButton, *buttonsBox;

	mainWindowData->mainBox = gtk_frame_new("Choisir un mode");
	buttonsBox = gtk_utils_create_buttonsbox(FALSE, 5, 140, 20,
			GTK_BUTTONBOX_SPREAD);

	// Boutons
	clientButton = gtk_button_new_with_mnemonic("_Client");
	serverButton = gtk_button_new_with_mnemonic("_Serveur");
	helpButton = gtk_button_new_with_mnemonic("_Règles");
	aboutButton = gtk_button_new_with_mnemonic("_A propos");
	quitButton = gtk_button_new_from_stock(GTK_STOCK_QUIT);

	// Association des fonctions a appeller aux boutons
	g_signal_connect(G_OBJECT(clientButton), "clicked",
			G_CALLBACK(main_window_client), mainWindowData);
	g_signal_connect(G_OBJECT(serverButton), "clicked",
			G_CALLBACK(main_window_server), mainWindowServerData);
	g_signal_connect(G_OBJECT(helpButton), "clicked",
			G_CALLBACK(main_window_help), mainWindowData->window);
	g_signal_connect(G_OBJECT(aboutButton), "clicked",
			G_CALLBACK(main_window_about), mainWindowData->window);
	g_signal_connect(G_OBJECT(quitButton), "clicked",
			G_CALLBACK(main_window_quit), mainWindowData);

	// Ajout des boutons dans la partie boutons
	gtk_container_add(GTK_CONTAINER(buttonsBox), clientButton);
	gtk_container_add(GTK_CONTAINER(buttonsBox), serverButton);
	gtk_container_add(GTK_CONTAINER(buttonsBox), helpButton);
	gtk_container_add(GTK_CONTAINER(buttonsBox), aboutButton);
	gtk_container_add(GTK_CONTAINER(buttonsBox), quitButton);

	// Ajout de  la partie boutons au launcher, et du launcher a la fenetre
	gtk_container_add(GTK_CONTAINER(mainWindowData->mainBox), buttonsBox);

	gtk_container_add(GTK_CONTAINER(mainWindowData->window),
			mainWindowData->mainBox);
	gtk_widget_show_all(mainWindowData->window);
}

/* Callback : bouton Aide */
void main_window_help(GtkWidget* button, gpointer p_data) {
	GtkWidget *window, *dialog, *scrolled_window;
//	WebKitWebView *web_view;
/*	window = (GtkWidget *) p_data;
	char url[256];
	sprintf(url, "file://%s/data/help/help.html", g_get_current_dir());

	// Creation du dialogue
	dialog = gtk_dialog_new_with_buttons(TITLE, GTK_WINDOW(window),
			GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_NONE,
			NULL);

	gtk_window_set_title(GTK_WINDOW(dialog), TITLE" - Règles");
	gtk_window_set_default_size(GTK_WINDOW(dialog), 500, 400);
	gtk_window_set_icon(GTK_WINDOW(dialog),
			gdk_pixbuf_new_from_file("data/interface/icon.png", NULL));

	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	web_view = WEBKIT_WEB_VIEW(webkit_web_view_new());
	webkit_web_view_open(WEBKIT_WEB_VIEW(web_view), url);

	gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(web_view));
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), scrolled_window);

	// Affichage du dialogue et destruction une fois le OK valide
	gtk_widget_show_all(dialog);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);*/
}

/* Callback : bouton A propos */
void main_window_about(GtkWidget* button, gpointer p_data) {
	GtkWidget *window, *dialog, *vbox1, *hbox, *icon, *vbox2, *lblTitle,
			*lblSub, *lblAuthors, *lblDesc;
	PangoFontDescription* font;

	window = (GtkWidget *) p_data;

	// Creation du dialogue
	dialog = gtk_dialog_new_with_buttons(TITLE, GTK_WINDOW(window),
			GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_NONE,
			NULL);

	gtk_window_set_title(GTK_WINDOW(dialog), TITLE" - A propos");
	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
	gtk_window_set_icon(GTK_WINDOW(dialog),
			gdk_pixbuf_new_from_file("data/interface/icon.png", NULL));

	font = pango_font_description_from_string("Monospace");

	vbox1 = gtk_vbox_new(FALSE, 0);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);

	// Image de gauche
	icon = gtk_image_new_from_file("data/interface/icon.png");

	vbox2 = gtk_vbox_new(FALSE, 0);

	pango_font_description_set_size(font, 14 * PANGO_SCALE);
	pango_font_description_set_weight(font, PANGO_WEIGHT_HEAVY);
	lblTitle = gtk_label_new("Mega Guitar Blindtest");
	gtk_widget_modify_font(lblTitle, font);

	pango_font_description_set_size(font, 12 * PANGO_SCALE);
	pango_font_description_set_weight(font, PANGO_WEIGHT_BOLD);
	lblSub = gtk_label_new("Projet de C - Esial 2010-2011");
	gtk_widget_modify_font(lblSub, font);

	pango_font_description_set_size(font, 10 * PANGO_SCALE);
	pango_font_description_set_weight(font, PANGO_WEIGHT_NORMAL);
	lblAuthors = gtk_label_new("Par Julien VAUBOURG && Guillaume GÉRARD");
	gtk_widget_modify_font(lblAuthors, font);

	lblDesc
			= gtk_label_new(
					"Description : Mega Guitar BLindtest dont le but est\nde trouver le plus d'écrans correspondant à un générique !\n\nJoueurs : 4 maximum\nÂge : 3 ans et +");
	gtk_widget_modify_font(lblDesc, font);

	// Ajout des elements au dialogue
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox1);

	gtk_container_add(GTK_CONTAINER(vbox1), hbox);

	gtk_container_add(GTK_CONTAINER(hbox), icon);
	gtk_container_add(GTK_CONTAINER(hbox), vbox2);

	gtk_container_add(GTK_CONTAINER(vbox2), lblTitle);
	gtk_container_add(GTK_CONTAINER(vbox2), lblSub);
	gtk_container_add(GTK_CONTAINER(vbox2), lblAuthors);

	gtk_container_add(GTK_CONTAINER(vbox1), lblDesc);

	// Affichage du dialogue et destruction une fois le OK valide
	gtk_widget_show_all(dialog);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
}

/* Force la mise a jour visuelle d'un widget. Astuce from
 * http://mail.gnome.org/archives/gtk-app-devel-list/2004-April/msg00307.html */
void main_window_refresh_widget(GtkWidget* widget) {
	gtk_widget_queue_draw(widget);

	while (gtk_events_pending())
		gtk_main_iteration();
}

void main_window_quit(GtkWidget* button, gpointer p_data) {
	MainWindowData* mainWindowData = (MainWindowData *) p_data;
	int answer;

	answer = gtk_utils_dialog_yesno(mainWindowData->window, TITLE,
			"Êtes vous sur de vouloir quitter "TITLE" ?");

	if (answer == GTK_RESPONSE_ACCEPT)
		gtk_main_quit();
}
