#include "main_window_server.h"

/* Modification de la fenetre du launcher pour devenir une fenetre de serveur */
void main_window_server(GtkWidget* button, gpointer p_data) {
	GtkWidget *statsFrame, *infoFrame, *scoresFrame, *buttonsBox;

	// Recuperation de la structure de donnees de la fenetre
	MainWindowServerData* mainWindowServerData =
			(MainWindowServerData *) p_data;
	MainWindowData* mainWindowData = mainWindowServerData->mainWindowData;

	gtk_window_set_title(GTK_WINDOW(mainWindowData->window), TITLE" - Server");
	gtk_container_remove(GTK_CONTAINER(mainWindowData->window),
			mainWindowData->mainBox);
	mainWindowData->mainBox = gtk_vbox_new(FALSE, 0);

	// Creations des labels
	mainWindowServerData->statsLabel = gtk_label_new(NULL);
	mainWindowServerData->scoresLabel = gtk_label_new(NULL);
	mainWindowServerData->infoLabel = gtk_label_new(
			"Veuillez lancer la détection des wiimotes.");

	// Creation des frames contenant les labels
	statsFrame = gtk_frame_new("Stats");
	scoresFrame = gtk_frame_new("Scores");
	infoFrame = gtk_frame_new("Infos");

	// Ajout des labels aux frames
	gtk_container_add(GTK_CONTAINER(statsFrame),
			mainWindowServerData->statsLabel);
	gtk_container_add(GTK_CONTAINER(scoresFrame),
			mainWindowServerData->scoresLabel);
	gtk_container_add(GTK_CONTAINER(infoFrame), mainWindowServerData->infoLabel);

	// Ajout des frames a la fenetre
	gtk_container_add(GTK_CONTAINER(mainWindowData->mainBox), statsFrame);
	gtk_container_add(GTK_CONTAINER(mainWindowData->mainBox), scoresFrame);
	gtk_container_add(GTK_CONTAINER(mainWindowData->mainBox), infoFrame);

	// Creation d'une partie boutons
	buttonsBox = gtk_utils_create_buttonsbox(TRUE, 10, 140, 20,
			GTK_BUTTONBOX_EDGE);
	gtk_container_set_border_width(GTK_CONTAINER(buttonsBox), 15);

	// Creation des boutons
	mainWindowServerData->wiimotesButton = gtk_button_new_with_mnemonic(
			"Détecter les _wiimotes");
	mainWindowServerData->startButton = gtk_button_new_with_mnemonic("_Lancer");
	mainWindowServerData->pauseButton = gtk_button_new_with_mnemonic("_Pause");
	mainWindowServerData->resetButton = gtk_button_new_with_mnemonic("_Reset");
	mainWindowServerData->quitButton
			= gtk_button_new_from_stock(GTK_STOCK_QUIT);

	// Association des fonction a declencher correspondant aux boutons
	g_signal_connect(G_OBJECT(mainWindowServerData->wiimotesButton), "clicked",
			G_CALLBACK(main_window_server_wiimotes), mainWindowServerData);
	g_signal_connect(G_OBJECT(mainWindowServerData->startButton), "clicked",
			G_CALLBACK(main_window_server_start), mainWindowServerData);
	g_signal_connect(G_OBJECT(mainWindowServerData->pauseButton), "clicked",
			G_CALLBACK(main_window_server_pause), mainWindowServerData);
	g_signal_connect(G_OBJECT(mainWindowServerData->resetButton), "clicked",
			G_CALLBACK(main_window_server_reset), mainWindowServerData);
	g_signal_connect(G_OBJECT(mainWindowServerData->quitButton), "clicked",
			G_CALLBACK(main_window_server_quit), mainWindowData);

	// Ajout des boutons dans la partie boutons
	gtk_container_add(GTK_CONTAINER(buttonsBox),
			mainWindowServerData->wiimotesButton);
	gtk_container_add(GTK_CONTAINER(buttonsBox),
			mainWindowServerData->startButton);
	gtk_container_add(GTK_CONTAINER(buttonsBox),
			mainWindowServerData->pauseButton);
	gtk_container_add(GTK_CONTAINER(buttonsBox),
			mainWindowServerData->resetButton);
	gtk_container_add(GTK_CONTAINER(buttonsBox),
			mainWindowServerData->quitButton);

	// Ajout de la partie boutons a la fenetre
	gtk_container_add(GTK_CONTAINER(mainWindowData->mainBox), buttonsBox);

	gtk_container_add(GTK_CONTAINER(mainWindowData->window),
			mainWindowData->mainBox);
	gtk_widget_show_all(mainWindowData->window);

	// Masquage des boutons qui ne s'afficheront que lorsque que le jeu sera
	// lance
	gtk_widget_set_visible(mainWindowServerData->pauseButton, 0);
	gtk_widget_set_visible(mainWindowServerData->resetButton, 0);
	gtk_widget_set_sensitive(mainWindowServerData->startButton, 0);

	// Initialisation des donnees utiles au serveur
	server_init(mainWindowData->clients, &mainWindowServerData->series,
			&mainWindowServerData->sock, &mainWindowServerData->clientsThread);

	// Lancement d'un thread de mise a jour des labels de la fenetre
	g_thread_create((void *) main_window_server_update, mainWindowServerData,
			FALSE, NULL);
}

/* Fonction threadee, de mise a jour des labels */
void main_window_server_update(void* p_data) {
	char statsText[128], scoresText[128];
	int p;

	// Recuperation des donnees
	MainWindowServerData* mainWindowServerData =
			(MainWindowServerData *) p_data;
	MainWindowData* mainWindowData = mainWindowServerData->mainWindowData;

	// Mises a jour tant que le jeu est en cours
	while (!MGBT_EXIT_SIGNAL) {

		// Label des stats (nb joueurs, nb clients)
		sprintf(
				statsText,
				"%d joueur%s\n%d client%s",
				mainWindowData->game->nb_players,
				mainWindowData->game->nb_players > 1 ? "s" : "",
				server_socket_count_clients(mainWindowData->clients),
				server_socket_count_clients(mainWindowData->clients) > 1 ? "s"
						: "");

		// Si au moins un joueur est enregistre
		if (mainWindowData->game->nb_players > 0) {

			// Initialisation pour la concatenation
			*scoresText = '\0';

			// Pour chaque joueur, on ajoute son id et son score
			for (p = 0; p < mainWindowData->game->nb_players; p++)
				sprintf(scoresText, "%sJoueur %d ::: %d point%s\n", scoresText,
						p + 1, mainWindowData->game->players[p].score,
						mainWindowData->game->players[p].score > 1 ? "s" : "");

			// Suppression du dernier retour a la ligne
			scoresText[strlen(scoresText) - 1] = '\0';
		} else
			strcpy(scoresText, "Aucun joueur.");

		gdk_threads_enter();

		gtk_label_set_text(GTK_LABEL(mainWindowServerData->statsLabel),
				statsText);
		gtk_label_set_text(GTK_LABEL(mainWindowServerData->scoresLabel),
				scoresText);

		// On force les labels a se mettre a jour visuellement
		main_window_refresh_widget(mainWindowServerData->statsLabel);
		main_window_refresh_widget(mainWindowServerData->scoresLabel);

		// Activation ou desactivation du bouton de lancement de partie,
		// selon si il y a au moins un joueur et un client de connectes
		gtk_widget_set_sensitive(
				mainWindowServerData->startButton,
				mainWindowData->game->nb_players > 0
						&& server_socket_count_clients(mainWindowData->clients)
								> 0);

		gdk_threads_leave();

		sleep(1);
	}
}

/* Callback : bouton Detection des wiimotes */
void main_window_server_wiimotes(GtkWidget* button, gpointer p_data) {
	char found[128];

	// Recuperation des donnees
	MainWindowServerData* mainWindowServerData =
			(MainWindowServerData *) p_data;
	MainWindowData* mainWindowData = mainWindowServerData->mainWindowData;

	// Mise a jour du label d'infos
	gtk_label_set_text(GTK_LABEL(mainWindowServerData->infoLabel),
			"Détection des wiimotes...");
	main_window_refresh_widget(mainWindowServerData->infoLabel);

	// Detection des wiimotes
	server_wiimote_init(mainWindowData->game);

	// Mise a jour du label d'infos
	sprintf(found,
			"%d joueur%s détecté%s\n(vous pouvez relancer la détection)",
			mainWindowData->game->nb_players,
			mainWindowData->game->nb_players > 1 ? "s" : "",
			mainWindowData->game->nb_players > 1 ? "s" : "");

	gtk_label_set_text(GTK_LABEL(mainWindowServerData->infoLabel), found);
	gtk_label_set_justify(GTK_LABEL(mainWindowServerData->infoLabel),
			GTK_JUSTIFY_CENTER);
}

/* Callback : bouton Lancer le jeu */
void main_window_server_start(GtkWidget* button, gpointer p_data) {
	MainWindowServerData* mainWindowServerData =
			(MainWindowServerData *) p_data;

	// Changement du set de boutons (apparition de Pause et Reset)
	gtk_widget_set_visible(mainWindowServerData->wiimotesButton, 0);
	gtk_widget_set_visible(mainWindowServerData->startButton, 0);
	gtk_widget_set_visible(mainWindowServerData->pauseButton, 1);
	gtk_widget_set_visible(mainWindowServerData->resetButton, 1);

	// Mise a jour du label d'infos
	gtk_label_set_text(GTK_LABEL(mainWindowServerData->infoLabel),
			"Jeu en cours...");

	// Lancement du thread du serveur
	g_thread_create((void *) main_window_server_start_run,
			mainWindowServerData, FALSE, NULL);
}

/* Fonction threadee, lancement du serveur */
void main_window_server_start_run(void* p_data) {
	MainWindowServerData* mainWindowServerData =
			(MainWindowServerData *) p_data;

	server(mainWindowServerData->mainWindowData->game,
			mainWindowServerData->mainWindowData->clients,
			mainWindowServerData->series, mainWindowServerData->sock,
			mainWindowServerData->clientsThread);
}

/* Callback : bouton Pause */
void main_window_server_pause(GtkWidget* button, gpointer p_data) {
	MainWindowServerData* mainWindowServerData =
			(MainWindowServerData *) p_data;

	// Si le bouton affichait Pause, on demande la pause et on change son
	// label en Reprise
	if (!strcmp(gtk_button_get_label(GTK_BUTTON(button)), "Pause")) {
		mainWindowServerData->mainWindowData->game->players[0].wiimoteStatus
				= W_REQ_PAUSE;
		gtk_button_set_label(GTK_BUTTON(button), "Reprise");

		// Si il affichait Reprise, on demande la reprise et on change son label
		// en Pause
	} else {
		mainWindowServerData->mainWindowData->game->players[0].wiimoteStatus
				= W_REPRISE;
		gtk_button_set_label(GTK_BUTTON(button), "Pause");
	}

	mainWindowServerData->mainWindowData->game->players[0].hasNewEvent = 1;
}

/* Callback : bouton reset */
void main_window_server_reset(GtkWidget* button, gpointer p_data) {
	MainWindowServerData* mainWindowServerData =
			(MainWindowServerData *) p_data;

	// Remise a zero de tous les scores
	server_score_reset(mainWindowServerData->mainWindowData->game);
}

/* Callback : bouton quit */
void main_window_server_quit(GtkWidget* button, gpointer p_data) {
	MainWindowData* mainWindowData =
			(MainWindowData *) p_data;
	int answer;

	answer = gtk_utils_dialog_yesno(mainWindowData->window,
			TITLE, "Êtes vous sur de vouloir quitter "TITLE" ?");

	// Si on confirme, on quitte tout
	if (answer == GTK_RESPONSE_ACCEPT)
		server_quit_all();
}
