#include "server.h"

/* Initialisation des donnees necessaires au serveur */
void server_init(Clients* clients, Series** series, SOCKET* sock, pthread_t* clientsThread) {
	*series = server_series_init();
	*sock = server_socket_init();
	*clientsThread = server_socket_welcome_clients_start(clients, *sock, *series);
}

/* Lancement du serveur, en lui passant les donnees renseignees par _init */
void server(GameStatus* game, Clients* clients, Series* series, SOCKET sock, pthread_t clientsThread) {
	pthread_t wiimoteThread, soundSerieThread, soundEventsThread;
	SoundData soundDataSerie, soundDataEvents;
	time_t timer;
	char* currentSerie;
	int p, gamePause = 0, gameChange = 1;

	// Lancement des threads
	wiimoteThread = server_wiimote_start(game);
	soundSerieThread = server_sound_start(&soundDataSerie);
	soundEventsThread = server_sound_start(&soundDataEvents);

	sleep(4);

	// Tant que le jeu ne cesse pas
	while (!MGBT_EXIT_SIGNAL) {

		// Si on demande un changement de generique, ou s'il y a timeout
		if(gameChange || (difftime(time(NULL), timer) >= TIMER && !gamePause)) {

			// Cas d'un timeout : un son est declenchee, et les
			// clients affichent la configuration adequate
			if (!gameChange) {
				server_sound_play_start_events(&soundDataEvents, "timer");
				server_setWindowClient_timerOut(clients);
				server_socket_send_windowClients(clients);
			}

			// Arret du generique en cours
			server_sound_play_stop(&soundDataSerie);

			sleep(1);

			// Choix d'un nouveau generique
			currentSerie = server_sound_random(series);

			// Compte a rebours avant le lancement du nouveau
			// generique : les wiimotes sont bloquees
			server_wiimote_lockEvents(game);
			server_reset_data(clients, game);
			server_setWindowClient_countdown_321(clients, &soundDataEvents);
			server_wiimote_unlockEvents(game);

			// Creation de nouvelles configurations pour les
			// fenetres des clients
			server_setWindowClient_play(clients, series);

			// Lancement du generique et envoi des configurations
			// aux clients pour qu'ils mettent leur fenetre a jour
			server_sound_play_start_serie(&soundDataSerie, currentSerie);
			server_socket_send_windowClients(clients);
			
			// Enregistrement du timer
			time(&timer);

			gameChange = 0;
		}

		// Recherche des joueurs qui ont un nouvel evenement a faire
		// traiter
		for(p = 0; p < game->nb_players && !MGBT_EXIT_SIGNAL; p++) {
			if(game->players[p].hasNewEvent) {

				// Si les wiimotes ne sont pas en pause, on
				// traite l'evenement
				if(game->players[p].wiimoteStatus != W_PAUSE) {
					switch(game->players[p].wiimoteStatus) {

						// Validation d'une couleur
						case W_WHAMMY_DOWN :
						case W_PLAY :

							// Mise a jour du score et envoi de l'evenement correspondant au
							// joueur. Si c'etait un coup de vibrato et qu'il etait justifie,
							// le retour indique qu'il faut passer a un nouveau generique
							gameChange = server_score(&(game->players[p]), clients, currentSerie, &soundDataEvents);
							server_score_display(game);
						break;

						// Demande de mise en pause
						case W_REQ_PAUSE :
							server_setWindowClient_pause(clients, game, &timer);
							server_socket_send_windowClients(clients);
							server_sound_play_pause(&soundDataSerie);

							gamePause = 1;
						break;

						// Demande de reprise de la pause
						case W_REPRISE :
							server_setWindowClient_reprise(clients, game, &timer);
							server_socket_send_windowClients(clients);
							server_sound_play_reprise(&soundDataSerie);

							gamePause = 0;
						break;

						// Un joueur souhaite que la partie finisse maintenant
						case W_QUIT :
							server_quit_all();
						break;
					}

					game->players[p].hasNewEvent = 0;
				}
			}
		}

		usleep(1);
	}

	// Creation et envoi des configurations pour demander a tous les clients
	// de quitter
	server_setWindowClient_quit(clients);
	server_socket_send_windowClients(clients);
	sleep(1);

	// Arret de tous les threads de son
	server_sound_stop(soundSerieThread);
	server_sound_stop(soundEventsThread);

	// Arret du thread d'accueil des nouveaux clients, ainsi que des
	// captures d'evenements wiimote
	server_socket_welcome_clients_stop(clientsThread);
	server_wiimote_stop(wiimoteThread);

	// Liberation memoire
	server_socket_free(clients);
	server_series_free(series);

	// Fermeture du socket server
	server_socket_close(sock);
}

/* Remise a zero des donnees pour un generique passe */
void server_reset_data(Clients* clients, GameStatus* game) {
	Client* client = clients->list;
	int p, c;

	// Suppression des evenements wiimotes enregistres
	for(p = 0; p < game->nb_players; p++) {
		game->players[p].hasNewEvent = 0;

		for(c = 0; c < game->nb_players; c++) {
			game->players[p].colors[c].nbStrums = 0;
			game->players[p].colors[c].validated = 0;
		}
	}

	// Mise a zero des configurations clients (sans leur envoyer)
	while (client != NULL) {
		memset(client->winClient, 0, sizeof(WindowClient));
		client = client->next;
	}
}

/* Quitter le jeu */
void server_quit_all() {
	MGBT_EXIT_SIGNAL = 1;
	gtk_main_quit();
}
