#include "server_score.h"

/* Mise a jour des scores des joueurs, en fonction de leurs evenements wiimote
 * et de l'etat des fenetres des clients */
int server_score(PlayerStatus* player, Clients* clients, char* currentSerie, SoundData* soundDataEvents) {
	Client* client;
	int c, next = 0;

	for(c = 0; c < NB_COLOR + 1; c++) {
		if(player->colors[c].validated) {
			client = clients->list;

			// Si la "couleur" est un coup de vibrato, on verifie si
			// celui-ci etait justifie ou non. Si c'est le cas, la
			// fonction renvoi vrai, et le jeu passe au prochain
			// generique. Mise a jour des scores selon si le vibrato
			// etait justifie ou non.
			if(c == G_WHAMMY_BAR)
				next = server_score_check_whammy_bar(clients, player, currentSerie, soundDataEvents);

			// Sinon, on verifie simplement si la validation de la
			// couleur entraine l'extinction ou non de la fenetre
			// d'un client. Si c'est le cas, le score est mis a jour
			// selon si la fenetre en question correspondait au
			// generique ou non.
			else
				server_score_check_color(clients, player, c, currentSerie, soundDataEvents);

			// Reset de l'action wiimote
			player->colors[c].validated = 0;
			player->colors[c].nbStrums = 0;
		}
	}

	return next;
}

/* Augmente ou diminue le score, selon si la couleur validee correspond a la
 * fenetre d'un client qui lui-meme correspond au generique en cours ou non. Si
 * une fenetre correspond, on demande a son client de la passer en rouge ou
 * vert. */
void server_score_check_color(Clients* clients, PlayerStatus* player, int c, char* currentSerie, SoundData* soundDataEvents) {
	Client* client = clients->list;
	char* colors[] = COLORS;

	// Pour chaque fenetre des clients
	while(client != NULL) {

		// Si la couleur et le nombre de coup de strum correspondent a
		// la fenetre d'un client
		if(client->winClient->numberStrum == player->colors[c].nbStrums
			&& !strcmp(client->winClient->color, colors[c])) {

			// Si c'est la meme serie, la fenetre passe vert on le
			// joueur gagne des points
			if(!strcmp(client->winClient->serieDir, currentSerie)) {
				player->score += SCORE_GOOD;
				server_score_good(client, clients, player, soundDataEvents);

			// Sinon rouge et il perd
			} else {
				player->score += SCORE_BAD;
				server_score_bad(client, clients, player, soundDataEvents);
			}
		}

		client = client->next;
	}
}

/* Si le vibrato a ete actionne alors qu'il restait des fenetres de client
 * correspondant a la serie du generique en cours, le joueur perd des points et
 * il ne se passe rien. Sinon il en gagne, et la fonction renvoi vrai pour
 * indiquer qu'il faut changer de generique. */
int server_score_check_whammy_bar(Clients* clients, PlayerStatus* player, char* currentSerie, SoundData* soundDataEvents) {
	int next= 0;

	// Reste-t-il des ecrans qui affichent la serie du generique actuel ?
	if(server_score_noMoreGood(clients, currentSerie)){
		player->score += SCORE_GOOD_WHAMMY_BAR;
		server_score_goodWhammyBar(clients, player, soundDataEvents);
		next = 1;

	} else  {
		player->score += SCORE_BAD_WHAMMY_BAR;
		server_score_badWhammyBar(player, soundDataEvents);
	}

	return next;
}

/* Indique si l'une des fenetres des clients affiche ou non encore la serie
 * correspondant au generique en cours */
int server_score_noMoreGood(Clients* clients, char* currentSerie) {
	Client* client = clients->list;
	int remaining = 0;

	while(client != NULL) {
		if(!strcmp(client->winClient->serieDir, currentSerie)
			&& !client->winClient->downBy)
			remaining++;

		client = client->next;
	}

	return remaining == 0;
}

/* Un coup de vibrato justifie a ete donne : on l'indique sur toutes les
 * fenetres des clients, en lui passant le nom du joueur responsable. */
void server_score_goodWhammyBar(Clients* clients, PlayerStatus* player, SoundData* soundDataEvents) {
	Client* client = clients->list;

	// Chaque client est informe
	while(client != NULL) {
		client->winClient->event = EVENT_GOOD_WHAMMY_BAR;
		client->winClient->downBy = (player->id) + 1;

		client = client->next;
	}

	// Evenements associes (son et vibration de la wiimote du joueur
	// responsable)
	server_sound_play_start_events(soundDataEvents, "switch");
	server_wiimote_good(player);
	server_socket_send_windowClients(clients);
}

/* Coup de vibrato injustifie : il ne se passe rien, sauf un son et un coup de
 * vibration au joueur responsable */
void server_score_badWhammyBar(PlayerStatus* player, SoundData* soundDataEvents) {
	server_sound_play_start_events(soundDataEvents, "bad");
	server_wiimote_badWhammyBar(player);
}

/* Un joueur vient de valider la fenetre d'un client qui correspondait bien a la
 * serie en cours : la fenetre s'affiche en vert en indique le nom du joueur
 * responsable */
void server_score_good(Client* client, Clients* clients, PlayerStatus* player, SoundData* soundDataEvents) {
	client->winClient->event = EVENT_GOOD;
	client->winClient->downBy = (player->id) + 1;

	// Un son est emis et le joueur recoi une gentille vibration dans sa
	// wiimote gagnante
	server_sound_play_start_events(soundDataEvents, "good");
	server_wiimote_good(player);
	server_socket_send_windowClient(client, clients);
}

/* Un joueur vient de valider la fenetre d'un client qui ne correspondait pas a
 * la serie en cours : la fenetre en question passe en rouge et indique le
 * joueur fautif */
void server_score_bad(Client* client, Clients* clients, PlayerStatus* player, SoundData* soundDataEvents) {
	client->winClient->event = EVENT_BAD;
	client->winClient->downBy = (player->id) + 1;

	// Un son est emis et un coup de vibration retentit
	server_sound_play_start_events(soundDataEvents, "bad");
	server_wiimote_bad(player);
	server_socket_send_windowClient(client, clients);
}

/* Affiche les scores sur le stdout */
void server_score_display(GameStatus* game) {
	int p;

	for(p = 0; p < game->nb_players; p++)
		printf("[SCORE] JOUEUR %d : %d points\n", (game->players[p].id) + 1, game->players[p].score);
}

/* Remet tous les scores a zero */
void server_score_reset(GameStatus* game) {
	int p;

	for(p = 0; p < game->nb_players; p++)
		game->players[p].score = 0;
}
