#include "server_series.h"

/* Initialise la structure de donnees contenant les informations sur les series
 * disponibles, en consultant le contenu du repertoire correspondant. */
Series* server_series_init() {
	char *serieDir, *slash, screensPath[256], soundPath[256];
	glob_t dirs, screens, sound;
	Series* series;
	int i;

	series = (Series *) malloc(sizeof(Series));

	// Recuperation de la liste des fichiers dans le repertoire des series
	if (!glob(DIR_SERIES, GLOB_MARK | GLOB_NOSORT, NULL, &dirs)) {
		series->list = (Serie **) malloc(sizeof(Serie *) * dirs.gl_pathc);
		series->nb_serie = 0;

		// Pour chacun de ces fichiers
		for (i = 0; i < dirs.gl_pathc; i++) {
			slash = &(dirs.gl_pathv[i][strlen(dirs.gl_pathv[i]) - 1]);

			// Si il fini par un slash, c'est que c'est un
			// repertoire
			if (*slash == '/') {

				// Suppression du slash final dans le nom du
				// repertoire
				*slash = '\0';
				serieDir = strrchr(dirs.gl_pathv[i], '/') + 1;

				// Si ce n'est pas un repertoire cache et que
				// son nom est inferieur a la taille de la
				// chaine dans laquel il sera stocke
				if (serieDir[0] != '.' && strlen(serieDir) < 256) {
					sprintf(screensPath, "%s/%s", dirs.gl_pathv[i], "screen_*.png");
					sprintf(soundPath, "%s/%s", dirs.gl_pathv[i], "sound.ogg");

					// Recherche de tous les fichiers
					// screen_*.png et du fichier sound.ogg
					if (!glob(screensPath, GLOB_MARK | GLOB_NOSORT, NULL, &screens)
						&& !glob(soundPath, GLOB_MARK | GLOB_NOSORT, NULL, &sound)) {

						// Ajout de la liste dans la
						// structure de donnees
						series->list[series->nb_serie] = (Serie *) malloc(sizeof(Serie));
						strcpy(series->list[series->nb_serie]->dir, serieDir);

						// Nombre de screens pour cette serie
						series->list[series->nb_serie]->nb_screen = screens.gl_pathc;
						series->nb_serie++;
					}
				}
			}
		}
	}

	globfree(&sound);
	globfree(&screens);
	globfree(&dirs);

	return series;
}

/* Liberation de l'espace alloue pour les series. */
void server_series_free(Series* series) {
	int i;

	for (i = 0; i < series->nb_serie; free(series->list[i++]))
		;
	free(series);
}
