#include "server_setWindowClient.h"

/* Configure le client pour qu'il positionne sa fenetre a l'etat initial
 * (affichage du logo). */
void server_setWindowClient_first(Client* client, Clients* clients, Series* series) {
	client->winClient = (WindowClient *) malloc(sizeof(WindowClient));
	memset(client->winClient, 0, sizeof(client->winClient));

	client->winClient->event = EVENT_FIRST;
}

/* Configure tous les clients (serie, couleur et nombre), en selectionnant leur
 * affichage de facon aleatoire et non redondante. */
void server_setWindowClient_play(Clients* clients, Series* series) {
	char* colors[] = COLORS;
	int serie;
	Client *client, *client_;
	struct timespec ts;

	client = clients->list;

	// Pour chaque client
	while (client != NULL) {
		client->winClient->numberStrum = 0;

		// Tant que le nombre de strums est a zero, cela signifie que
		// ses donnees n'ont pas encore ete determinee
		while (!client->winClient->numberStrum) {
			clock_gettime(CLOCK_REALTIME, &ts);
			srand(ts.tv_sec + ts.tv_nsec);

			// Une serie au hasard
			serie = rand() % series->nb_serie;

			strcpy(client->winClient->serieDir, series->list[serie]->dir);
			strcpy(client->winClient->color, colors[rand() % NB_COLOR]);

			// Un ecran de la serie au hasard
			client->winClient->screenNumber = rand() % series->list[serie]->nb_screen;

			// Un nombre de strum au hasard
			client->winClient->numberStrum = rand() % NB_STRUM_MAX;

			client_ = clients->list;

			// Puis, on verifie parmis les clients qui ont deja eu
			// leur mise a jour, si cette configuration de fenetre
			// n'a pas deja ete donnee
			while (client_ != client) {

				// Si c'est le cas, on remet le nombre de strums
				// a zero pour relancer la configuration
				if ((!strcmp(client->winClient->serieDir, client_->winClient->serieDir)
						&& client->winClient->screenNumber == client_->winClient->screenNumber)
						|| (client->winClient->numberStrum == client_->winClient->numberStrum
						&& !strcmp(client->winClient->color, client_->winClient->color)))
					client->winClient->numberStrum = 0;

				client_ = client_->next;
			}
		}

		client->winClient->event = EVENT_PLAY;
		client = client->next;
	}
}

/* Configure les clients (et leur envoi leur configuration) pour qu'ils
 * affichent 3 puis 2 puis 1, avec un son pour chaque du cote serveur. */
void server_setWindowClient_countdown_321(Clients* clients, SoundData* soundDataEvents) {
	int i;

	for (i = COUNTDOWN; i > 0; i--) {
		server_sound_play_start_events(soundDataEvents, "countdown");
		server_setWindowClient_countdown(clients, i);
		server_socket_send_windowClients(clients);

		sleep(1);
	}
}

/* Configure les clients pour qu'ils se mettent en mode countdown, en specifiant
 * le chiffre a afficher. */
void server_setWindowClient_countdown(Clients* clients, int step) {
	Client* client = clients->list;

	while (client != NULL) {
		client->winClient->screenNumber = step;
		client->winClient->event = EVENT_COUNTDOWN;
		client = client->next;
	}
}

/* Configure les clients pour afficher un ecran de time out. */
void server_setWindowClient_timerOut(Clients* clients) {
	Client* client = clients->list;

	while (client != NULL) {
		client->winClient->event = EVENT_TIMER;
		client = client->next;
	}
}

/* Configure les clients pour leur indiquer d'afficher un ecran de pause. Le
 * temps ecoule est enregistre dans la variable de temps. Les wiimotes des
 * joueurs sont aussi mises en mode pause, pour que leurs evenements ne soient
 * plus pris en compte (sauf le relachement de la pause). */
void server_setWindowClient_pause(Clients* clients, GameStatus* game, time_t* timer) {
	Client* client = clients->list;
	int p;

	// Tous les clients affichent pause
	while (client != NULL) {
		client->winClient->event = EVENT_PAUSE;
		client = client->next;
	}

	// Toutes les wiimotes n'acceptent plus que l'ordre de reprise
	for(p = 0; p < game->nb_players; p++)
		game->players[p].wiimoteStatus = W_PAUSE;

	// Enregistrement du temps ecoule
	*timer = difftime(time(NULL), *timer);
}

/* Configure les clients pour leur indiquer de quitter leur ecran de pause et
 * d'afficher ce qui est toujours dans le configuration. Indique aux wiimotes
 * que les evenements peuvent etre repris en compte. */
void server_setWindowClient_reprise(Clients* clients, GameStatus* game, time_t* timer) {
	Client* client = clients->list;
	int p;

	// Tous les clients reaffichent leur configuration d'avant la pause
	while (client != NULL) {
		client->winClient->event = EVENT_PLAY;
		client = client->next;
	}

	// Les wiimotes sont "reactivees"
	for(p = 0; p < game->nb_players; p++)
		game->players[p].wiimoteStatus = W_PLAY;

	// Le temps ecoule est pris en compte pour revenir a la situation
	// d'avant-pause
	*timer = time(NULL) - *timer;
}

/* Configure les clients pour leur demander de se fermer. */
void server_setWindowClient_quit(Clients* clients) {
	Client* client = clients->list;

	while (client != NULL) {
		client->winClient->event = EVENT_QUIT;
		client = client->next;
	}
}
